@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2009  The DOSBox Team
+ *  Copyright (C) 2002-2010  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: setup.h,v 1.41 2009/05/27 09:15:41 qbix79 Exp $ */
+/* $Id: setup.h,v 1.41 2009-05-27 09:15:41 qbix79 Exp $ */
 
 #ifndef DOSBOX_SETUP_H
 #define DOSBOX_SETUP_H
@@ -42,6 +42,11 @@
 #include <string>
 #endif
 
+#ifndef CH_CSTDIO
+#define CH_CSTDIO
+#include <cstdio>
+#endif
+
 
 class Hex {
 private:
@@ -85,26 +90,26 @@ public:
 	Value(std::string const& in,Etype _t) :_string(0),type(V_NONE) {SetValue(in,_t);}
 	
 	/* Assigment operators */
-	Value& operator= (Hex in) throw(WrongType)                { return copy(Value(in));}
-	Value& operator= (int in) throw(WrongType)                { return copy(Value(in));}
-	Value& operator= (bool in) throw(WrongType)               { return copy(Value(in));}
-	Value& operator= (double in) throw(WrongType)             { return copy(Value(in));}
-	Value& operator= (std::string const& in) throw(WrongType) { return copy(Value(in));}
-	Value& operator= (char const * const in) throw(WrongType) { return copy(Value(in));}
-	Value& operator= (Value const& in) throw(WrongType)       { return copy(Value(in));}
+	Value& operator= (Hex in)                 { return copy(Value(in));}
+	Value& operator= (int in)                 { return copy(Value(in));}
+	Value& operator= (bool in)                { return copy(Value(in));}
+	Value& operator= (double in)              { return copy(Value(in));}
+	Value& operator= (std::string const& in)  { return copy(Value(in));}
+	Value& operator= (char const * const in)  { return copy(Value(in));}
+	Value& operator= (Value const& in)        { return copy(Value(in));}
 
 	bool operator== (Value const & other);
-	operator bool () const throw(WrongType);
-	operator Hex () const throw(WrongType);
-	operator int () const throw(WrongType);
-	operator double () const throw(WrongType);
-	operator char const* () const throw(WrongType);
-	void SetValue(std::string const& in,Etype _type = V_CURRENT) throw(WrongType);
+	operator bool () const;
+	operator Hex () const;
+	operator int () const;
+	operator double () const;
+	operator char const* () const;
+	void SetValue(std::string const& in,Etype _type = V_CURRENT);
 	std::string ToString() const;
 
 private:
 	void destroy() throw();
-	Value& copy(Value const& in) throw(WrongType);
+	Value& copy(Value const& in);
 	void plaincopy(Value const& in) throw();
 	void set_hex(std::string const& in);
 	void set_int(std::string const&in);
